function updateItemState(itemId, isChecked) {
  const checkbox = document.querySelector(`#item-${itemId} input[type='checkbox']`);
  if (checkbox) {
    checkbox.checked = isChecked;
    checkbox.disabled = false;
    const li = checkbox.closest('li');
    li.classList.remove('opacity-50', 'bg-light', 'text-dark', 'bg-success', 'text-white');

    if (isChecked) {
      li.classList.add('bg-success', 'text-white');
    } else {
      li.classList.add('item-not-checked');
    }

    const sp = li.querySelector('.spinner-border');
    if (sp) sp.remove();
  }
  updateProgressBar();
}

function updateProgressBar() {
  const items = document.querySelectorAll('#items li');
  const total = items.length;
  const purchased = Array.from(items).filter(li => li.classList.contains('bg-success')).length;
  const percent = total > 0 ? Math.round((purchased / total) * 100) : 0;

  const progressBar = document.getElementById('progress-bar');
  if (progressBar) {
    progressBar.style.width = `${percent}%`;
    progressBar.setAttribute('aria-valuenow', percent);
    progressBar.textContent = `${percent}%`;
  }
}

function addItem(listId) {
  const name = document.getElementById('newItem').value;
  const quantityInput = document.getElementById('newQuantity');
  let quantity = 1;

  if (quantityInput) {
    quantity = parseInt(quantityInput.value);
    if (isNaN(quantity) || quantity < 1) {
      quantity = 1;
    }
  }

  if (name.trim() === '') return;

  socket.emit('add_item', { list_id: listId, name: name, quantity: quantity });

  document.getElementById('newItem').value = '';
  if (quantityInput) quantityInput.value = 1;
  document.getElementById('newItem').focus();
}

function deleteItem(id) {
  if (confirm('Na pewno usunąć produkt?')) {
    socket.emit('delete_item', { item_id: id });
  }
}

function editItem(id, oldName, oldQuantity) {
  const newName = prompt('Podaj nową nazwę (lub zostaw starą):', oldName);
  if (newName === null) return;

  const newQuantityStr = prompt('Podaj nową ilość:', oldQuantity);
  if (newQuantityStr === null) return;

  const finalName = newName.trim() !== '' ? newName.trim() : oldName;

  let newQuantity = parseInt(newQuantityStr);
  if (isNaN(newQuantity) || newQuantity < 1) {
    newQuantity = oldQuantity;
  }

  socket.emit('edit_item', { item_id: id, new_name: finalName, new_quantity: newQuantity });
}

function submitExpense(listId) {
  const amountInput = document.getElementById('expenseAmount');
  const amount = parseFloat(amountInput.value);
  if (isNaN(amount) || amount <= 0) {
    showToast('Podaj poprawną kwotę!');
    return;
  }
  socket.emit('add_expense', {
    list_id: listId,
    amount: amount
  });
  amountInput.value = '';
}

function copyLink(link) {
  if (navigator.clipboard && window.isSecureContext) {
    navigator.clipboard.writeText(link).then(() => {
      showToast('Link skopiowany do schowka!');
    }).catch((err) => {
      console.error('Błąd clipboard API:', err);
      fallbackCopyText(link);
    });
  } else {
    fallbackCopyText(link);
  }
}

/* function shareLink(link) {
  if (navigator.share) {
    navigator.share({
      title: 'Udostępnij moją listę',
      text: 'Zobacz tę listę!',
      url: link
    })
    .catch((error) => {
      console.error('Błąd podczas udostępniania', error);
      alert('Nie udało się udostępnić linka');
    });
  } else {
    copyLink(link);
  }
}

function fallbackCopy(link) {
  navigator.clipboard.writeText(link).then(() => {
    alert('Link skopiowany do schowka!');
  });
}
 */

function openList(link) {
  window.open(link, '_blank');
}

function fallbackCopyText(text) {
  const textarea = document.createElement('textarea');
  textarea.value = text;
  textarea.style.position = 'fixed';
  textarea.style.top = 0;
  textarea.style.left = 0;
  textarea.style.opacity = 0;
  document.body.appendChild(textarea);
  textarea.focus();
  textarea.select();

  try {
    const successful = document.execCommand('copy');
    if (successful) {
      showToast('Link skopiowany do schowka!');
    } else {
      showToast('Nie udało się skopiować linku', 'warning');
    }
  } catch (err) {
    console.error('Fallback błąd kopiowania:', err);
    showToast('Nie udało się skopiować linku', 'warning');
  }

  document.body.removeChild(textarea);
}

function toggleVisibility(listId) {
  fetch('/toggle_visibility/' + listId, {method: 'POST'})
    .then(response => response.json())
    .then(data => {
      const shareHeader = document.getElementById('share-header');
      const shareUrlSpan = document.getElementById('share-url');
      const copyBtn = document.getElementById('copyBtn');
      const toggleBtn = document.getElementById('toggleVisibilityBtn');

      if (data.is_public) {
        shareHeader.textContent = '🔗 Udostępnij link:';
        shareUrlSpan.style.display = 'inline';
        shareUrlSpan.textContent = data.share_url;
        copyBtn.disabled = false;
        toggleBtn.innerHTML = '🙈 Ukryj listę';
      } else {
        shareHeader.textContent = '🙈 Lista jest ukryta przed gośćmi';
        shareUrlSpan.style.display = 'none';
        copyBtn.disabled = true;
        toggleBtn.innerHTML = '👁️ Udostępnij ponownie';
      }
    });
}

function showToast(message, type = 'primary') {
  const toastContainer = document.getElementById('toast-container');
  const toast = document.createElement('div');
  toast.className = `toast align-items-center text-bg-${type} border-0 show`;
  toast.setAttribute('role', 'alert');
  toast.innerHTML = `<div class="d-flex"><div class="toast-body">${message}</div></div>`;
  toastContainer.appendChild(toast);
  setTimeout(() => { toast.remove(); }, 1750);
}

function isListDifferent(oldItems, newItems) {
  if (oldItems.length !== newItems.length) return true;

  const oldIds = Array.from(oldItems).map(li => parseInt(li.id.replace('item-', ''), 10)).sort();
  const newIds = newItems.map(i => i.id).sort();

  for (let i = 0; i < newIds.length; i++) {
    if (oldIds[i] !== newIds[i]) {
      return true;
    }
  }
  return false;
}

function updateListSmoothly(newItems) {
  const itemsContainer = document.getElementById('items');
  const existingItemsMap = new Map();

  Array.from(itemsContainer.querySelectorAll('li')).forEach(li => {
    const id = parseInt(li.id.replace('item-', ''), 10);
    existingItemsMap.set(id, li);
  });

  const fragment = document.createDocumentFragment();

  newItems.forEach(item => {
    let li = existingItemsMap.get(item.id);
    let quantityBadge = '';
    if (item.quantity && item.quantity > 1) {
      quantityBadge = `<span class="badge bg-secondary">x${item.quantity}</span>`;
    }

    if (li) {
      const checkbox = li.querySelector('input[type="checkbox"]');
      if (checkbox) {
        checkbox.checked = item.purchased;
        checkbox.disabled = false;
      }

      li.classList.remove('bg-success', 'text-white', 'item-not-checked', 'opacity-50');
      if (item.purchased) {
        li.classList.add('bg-success', 'text-white');
      } else {
        li.classList.add('item-not-checked');
      }

      const nameSpan = li.querySelector(`#name-${item.id}`);
      const expectedName = `${item.name} ${quantityBadge}`.trim();
      if (nameSpan && nameSpan.innerHTML.trim() !== expectedName) {
        nameSpan.innerHTML = expectedName;
      }

      let noteEl = li.querySelector('small');
      if (item.note) {
        if (!noteEl) {
          const newNote = document.createElement('small');
          newNote.className = 'text-danger ms-4';
          newNote.innerHTML = `[ <b>${item.note}</b> ]`;
          nameSpan.insertAdjacentElement('afterend', newNote);
        } else {
          noteEl.innerHTML = `[ <b>${item.note}</b> ]`;
        }
      } else if (noteEl) {
        noteEl.remove();
      }

      const sp = li.querySelector('.spinner-border');
      if (sp) sp.remove();

    } else {
      li = document.createElement('li');
      li.className = `list-group-item d-flex justify-content-between align-items-center flex-wrap ${item.purchased ? 'bg-success text-white' : 'item-not-checked'}`;
      li.id = `item-${item.id}`;

      li.innerHTML = `
        <div class="d-flex align-items-center gap-3 flex-grow-1">
          <input class="large-checkbox" type="checkbox" ${item.purchased ? 'checked' : ''}>
          <span id="name-${item.id}" class="text-white">${item.name} ${quantityBadge}</span>
          ${item.note ? `<small class="text-danger ms-4">[ <b>${item.note}</b> ]</small>` : ''}
        </div>
        <button type="button" class="btn btn-sm btn-outline-info" onclick="openNoteModal(event, ${item.id})">📝</button>
      `;
    }

    fragment.appendChild(li);
  });

  itemsContainer.innerHTML = '';
  itemsContainer.appendChild(fragment);

  updateProgressBar();
  toggleEmptyPlaceholder();
}

document.addEventListener("DOMContentLoaded", function() {
  const receiptSection = document.getElementById("receiptSection");
  const toggleBtn = document.querySelector('[data-bs-target="#receiptSection"]');

  if (!receiptSection || !toggleBtn) return;

  if (localStorage.getItem("receiptSectionOpen") === "true") {
    new bootstrap.Collapse(receiptSection, { toggle: true });
  }

  receiptSection.addEventListener('shown.bs.collapse', function () {
    localStorage.setItem("receiptSectionOpen", "true");
  });

  receiptSection.addEventListener('hidden.bs.collapse', function () {
    localStorage.setItem("receiptSectionOpen", "false");
  });
});
